@@ -13,7 +13,7 @@
  */
 /*
  * Copyright (C) 1999-2001  Richard Hult
- * Copyright (C) 2003,2006  LUGFI
+ * Copyright (C) 2003,2006  Ricardo Markiewicz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -211,7 +211,7 @@ main (int argc, char **argv)
 					tmp = g_filename_display_basename (startup_files[i]);
 					filename = g_strdup_printf ("%s.%s", tmp, ext);
 					g_print ("Converting %s to %s ...\n", startup_files[i], filename);
-					schematic_export (new_schematic, filename, convert_width, convert_height, 0, format);
+					schematic_export (new_schematic, filename, convert_width, convert_height, 0, 1, format);
 					g_object_unref (G_OBJECT (new_schematic));
 					g_free (filename);
 					g_free (tmp);
