@@ -7,7 +7,7 @@
  *
  * Web page: http://arrakis.lug.fi.uba.ar/
  *
- * Copyright (C) 2004-2006  Andres de Barbara
+ * Copyright (C) 2004-2008 Ricardo Markiewicz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,7 +33,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <gnome.h>
-#include <gtksourceview/gtksourceprintjob.h>
+#include <gtksourceview/gtksourceprintcompositor.h>
 
 static void netlist_editor_finalize (GObject *object);
 static void netlist_editor_dispose (GObject *object);
@@ -47,7 +47,7 @@ struct _NetlistEditorPriv {
 	SchematicView *sv;
 	gchar * font;
 	GdkColor bgcolor, selectcolor, textcolor;
-	GtkSourceLanguagesManager *lm;
+	GtkSourceLanguageManager *lm;
 	GtkTextView *view;
 	GtkSourceBuffer *buffer;
 	GtkWindow *toplevel;
@@ -281,14 +281,41 @@ netlist_editor_save (GtkWidget * widget, NetlistEditor * nle)
 	}
 }
 
+/* This method append OREGANO_LANGDIR directory where the netlist.lang file
+ * is located to the search path of GtkSourceLanguageManager.
+ */
+void
+setup_language_manager_path(GtkSourceLanguageManager *lm)
+{
+	gchar **lang_files;
+	int i, lang_files_count;
+	char **new_langs;
+
+	lang_files = g_strdupv (gtk_source_language_manager_get_search_path (lm));
+
+	lang_files_count = g_strv_length (lang_files);
+	new_langs = g_new (char*, lang_files_count + 2);
+
+	for (i = 0; lang_files[i]; i++)
+		new_langs[i] = lang_files[i];
+
+	new_langs[lang_files_count] = g_strdup (OREGANO_LANGDIR);
+	new_langs[lang_files_count+1] = NULL;
+
+	g_free (lang_files);
+
+	gtk_source_language_manager_set_search_path (lm, new_langs);
+}
+
 NetlistEditor *
 netlist_editor_new (GtkSourceBuffer * textbuffer) {
+	gchar** lang_files;
 	NetlistEditor * nle;
 	GladeXML * gui;
 	GtkWidget * toplevel;
 	GtkScrolledWindow * scroll;
 	GtkSourceView * source_view;
-	GtkSourceLanguagesManager * lm;
+	GtkSourceLanguageManager * lm;
 	GtkButton * save, * sim, * close, * print;
 	GtkSourceLanguage *lang=NULL;
 	const GSList *list;
@@ -319,24 +346,21 @@ netlist_editor_new (GtkSourceBuffer * textbuffer) {
 	scroll = GTK_SCROLLED_WINDOW (glade_xml_get_widget (gui, "netlist-scrolled-window"));
 	
 	source_view = GTK_SOURCE_VIEW (gtk_source_view_new ());
-	lm = GTK_SOURCE_LANGUAGES_MANAGER (gtk_source_languages_manager_new ());
-	
-	g_object_set_data_full (G_OBJECT (source_view), "languages-manager",
+	lm = GTK_SOURCE_LANGUAGE_MANAGER (gtk_source_language_manager_new ());
+
+	setup_language_manager_path (lm);
+
+	g_object_set_data_full (G_OBJECT (source_view), "language-manager",
 		lm, (GDestroyNotify) g_object_unref);
 
-	list = gtk_source_languages_manager_get_available_languages (lm);
-	while (list) {
-		GtkSourceLanguage *l = (GtkSourceLanguage *)list->data;
-		if (!g_strcasecmp(gtk_source_language_get_name (l), "netlist")) {
-			lang = l;
-			break;
-		}
-		list = list->next;
-	}
+	lang = gtk_source_language_manager_get_language (lm, "netlist");
 
 	if (lang) {
 		gtk_source_buffer_set_language (GTK_SOURCE_BUFFER (textbuffer), lang);
-		gtk_source_buffer_set_highlight (GTK_SOURCE_BUFFER (textbuffer), TRUE);
+		gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (textbuffer), TRUE);
+		gtk_source_buffer_set_highlight_matching_brackets (GTK_SOURCE_BUFFER (textbuffer), TRUE);
+	} else {
+		g_warning ("Can't load netlist.lang in %s", OREGANO_LANGDIR "/netlist.lang");
 	}
 
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (source_view), TRUE);
