@@ -10,7 +10,7 @@
  * Web page: http://arrakis.lug.fi.uba.ar/
  *
  * Copyright (C) 1999-2001  Richard Hult
- * Copyright (C) 2003,2006  LUGFI
+ * Copyright (C) 2003,2006  Ricardo Markiewicz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -484,7 +484,7 @@ part_browser_create (SchematicView *schematic_view)
 	br = g_new0 (Browser, 1);
 	br->preview = NULL;
 	br->schematic_view = schematic_view;
-	br->hidden = TRUE;
+	br->hidden = FALSE;
 
 	schematic_view_set_browser(schematic_view, br);
 
