@@ -10,7 +10,7 @@
  * Web page: http://arrakis.lug.fi.uba.ar/
  *
  * Copyright (C) 1999-2001  Richard Hult
- * Copyright (C) 2003,2006  LUGFI
+ * Copyright (C) 2003,2006  Ricardo Markiewicz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -56,7 +56,6 @@ static void part_item_destroy(GtkObject *object);
 static void part_item_moved(SheetItem *sheet_item);
 
 static void edit_properties (SheetItem *object);
-static void properties_cmd (GtkWidget *widget, SchematicView *sv);
 
 static void selection_changed (PartItem *item, gboolean select,
 	gpointer user_data);
@@ -122,6 +121,7 @@ struct _PartItemPriv {
 typedef struct {
 	GtkDialog *dialog;
 	PartItem  *part_item;
+
 	/* List of GtkEntry's */
 	GList *widgets;
 } PartPropDialog;
@@ -129,21 +129,12 @@ typedef struct {
 static PartPropDialog *prop_dialog = NULL;
 static SheetItemClass *parent_class = NULL;
 
-/*
- * This is the lower part of the object popup menu. It contains actions
- * that are specific for parts.
- */
-static GnomeUIInfo part_popup_menu [] = {
-	GNOMEUIINFO_SEPARATOR,
-
-	{
-		GNOME_APP_UI_ITEM,
-		N_("Properties"), N_("Edit the part's properties"),
-		properties_cmd, NULL, NULL, GNOME_APP_PIXMAP_STOCK,
-		GTK_STOCK_PROPERTIES, 0, 0
-	},
-	GNOMEUIINFO_END
-};
+static const char *part_item_context_menu =
+"<ui>"
+"  <popup name='ItemMenu'>"
+"    <menuitem action='ObjectProperties'/>"
+"  </popup>"
+"</ui>";
 
 enum {
 	ANCHOR_NORTH,
@@ -192,6 +183,8 @@ GtkAnchorType part_item_get_anchor_from_part (Part *part)
 		return GTK_ANCHOR_SOUTH_WEST;
 	if ((anchor_v == ANCHOR_EAST) && (anchor_h == ANCHOR_SOUTH))
 		return GTK_ANCHOR_SOUTH_EAST;
+
+	return GTK_ANCHOR_SOUTH_EAST;
 }
 
 GType
@@ -230,7 +223,6 @@ part_item_class_init (PartItemClass *part_item_class)
 	sheet_item_class = SHEET_ITEM_CLASS(part_item_class);
 	parent_class = g_type_class_peek_parent(part_item_class);
 
-
 	object_class->set_property = part_item_set_property;
 	object_class->get_property = part_item_get_property;
 
@@ -291,11 +283,6 @@ part_item_class_init (PartItemClass *part_item_class)
 
 	sheet_item_class->place = part_item_place;
 	sheet_item_class->place_ghost = part_item_place_ghost;
-
-	sheet_item_class->context_menu = g_new0 (SheetItemMenu, 1);
-	sheet_item_class->context_menu->menu = part_popup_menu;
-	sheet_item_class->context_menu->size =
-		sizeof (part_popup_menu) / sizeof (part_popup_menu[0]);
 }
 
 static void
@@ -309,6 +296,8 @@ part_item_init (PartItem *item)
 	priv->cache_valid = FALSE;
 
 	item->priv = priv;
+
+	sheet_item_add_menu (SHEET_ITEM (item), part_item_context_menu);
 }
 
 static void
@@ -510,7 +499,7 @@ part_item_update_node_label (PartItem *item)
 {
 	PartItemPriv *priv;
 	Part *part;
-	GSList *labels, *label_items;
+	GSList *labels;
 	GnomeCanvasItem *canvas_item;
 	Pin *pins;
 	gint num_pins, i;
@@ -590,10 +579,9 @@ static void
 edit_properties_punta (PartItem *item)
 {
 	GSList *properties;
-	Sheet *sheet;
 	PartItemPriv *priv;
 	Part *part;
-	char *msg, *value, *name;
+	char *msg;
 	GladeXML *gui;
 	GtkRadioButton *radio_v, *radio_c;
 	GtkRadioButton *ac_r, *ac_m, *ac_i, *ac_p;
@@ -720,16 +708,15 @@ static void
 edit_properties (SheetItem *object)
 {
 	GSList *properties;
-	Sheet *sheet;
 	PartItem *item;
 	PartItemPriv *priv;
 	Part *part;
-	char *internal, *msg, *value, *name;
+	char *internal, *msg;
 	GladeXML *gui;
 	GtkTable *prop_table;
 	GtkNotebook *notebook;
 	gint response, y = 0;
-	gboolean got_iter, has_model;
+	gboolean has_model;
 	gchar *model_name = NULL;
 
 	g_return_if_fail (object != NULL);
@@ -865,29 +852,27 @@ edit_properties (SheetItem *object)
 	gtk_widget_destroy (GTK_WIDGET (prop_dialog->dialog));
 }
 
-static void
-properties_cmd (GtkWidget *widget, SchematicView *sv)
-{
-	GList *list;
-
-	list = schematic_view_get_selection (sv);
-	if ((list != NULL) && IS_PART_ITEM (list->data))
-		edit_properties (list->data);
+/* Matrix Multiplication
+   Tried art_affine_multiply, but that didn't work */
+static void matrix_mult(double* a,double* b,double* c){
+  c[0]=a[0]*b[0]+a[1]*b[2];
+  c[2]=a[2]*b[0]+a[3]*b[2];
+  c[1]=a[0]*b[1]+a[1]*b[3];
+  c[3]=a[2]*b[1]+a[3]*b[3];
 }
 
 static void
 part_rotated_callback (ItemData *data, int angle, SheetItem *sheet_item)
 {
 	double affine[6];
-	double x1, y1, x2, y2, x0, y0;
-	double left, top, right, bottom, dx, dy;
+	double affine_scale[6];
+	double affine_rotate[6];
+
 	GList *list;
 	GSList *label_items;
 	GtkAnchorType anchor;
-	ArtPoint src, dst;
 	GnomeCanvasGroup *group;
 	GnomeCanvasItem *canvas_item;
-	ArtPoint *old = NULL;
 	PartItem *item;
 	PartItemPriv *priv;
 	Part *part;
@@ -901,7 +886,28 @@ part_rotated_callback (ItemData *data, int angle, SheetItem *sheet_item)
 
 	priv = item->priv;
 
-	art_affine_rotate (affine, angle);
+	if (angle!=0) {
+	// check if the part is flipped
+	  if ((part->priv->flip & 1) && (part->priv->flip & 2)) {
+	    // Nothing to do in this case
+	    art_affine_rotate (affine, angle);
+	  } else {
+	    if ((part->priv->flip & 1) || (part->priv->flip & 2)) {
+	      // mirror over point (0,0)
+	      art_affine_scale(affine_scale,-1,-1);
+	      art_affine_rotate(affine_rotate,angle);
+	      /* Matrix multiplication */
+	      /* Don't use art_affine_multiply() here !*/
+	      matrix_mult(affine_rotate,affine_scale,affine);
+	      affine[4]=affine_rotate[4];
+	      affine[5]=affine_rotate[5];
+	    } else
+	      art_affine_rotate (affine, angle);
+	  }
+	} else {
+	  // angle==0: Nothing has changed, therefore do nothing
+	  art_affine_scale(affine,1,1);
+	}
 
 	for (list = group->item_list; list; list = list->next) {
 		canvas_item = GNOME_CANVAS_ITEM (list->data);
@@ -969,9 +975,10 @@ part_flipped_callback (ItemData *data, gboolean horizontal,
 	PartItemPriv *priv;
 	Part *part;
 	IDFlip flip;
-	double affine[6], x1, y1, x2, y2, left, top, right, bottom;
-	ArtPoint src, dst;
-	gdouble text_heigth, text_width;
+	double affine[6];
+	double affine_rotate[6];
+	double affine_scale[6];
+	int angle;
 
 	g_return_if_fail (sheet_item != NULL);
 	g_return_if_fail (IS_PART_ITEM (sheet_item));
@@ -988,6 +995,27 @@ part_flipped_callback (ItemData *data, gboolean horizontal,
 	else
 		art_affine_scale (affine, 1, -1);
 
+	angle=part_get_rotation(part);
+	if (angle!=0) {
+	  switch (angle){
+	  case 90: art_affine_rotate(affine_rotate,180);
+	    memcpy(affine_scale,affine,6*sizeof(double));
+	    break;
+	  case 180: //don't do a thing
+	    art_affine_rotate(affine_rotate,0);
+	    memcpy(affine_scale,affine,6*sizeof(double));
+	    break;
+	  case 270: art_affine_rotate(affine_rotate,0);
+	    // only to reset the memory of affine_scale, just a hack
+	    memcpy(affine_scale,affine,6*sizeof(double));
+	    // switch the flipping from horizontal to vertical and vice versa
+	    affine_scale[0]=affine[3];
+	    affine_scale[3]=affine[0];
+	  }
+	  matrix_mult(affine_scale,affine_rotate,affine);
+	}
+
+
 	for (list = group->item_list; list; list = list->next) {
 		canvas_item = GNOME_CANVAS_ITEM (list->data);
 
@@ -1024,6 +1052,123 @@ part_flipped_callback (ItemData *data, gboolean horizontal,
 	priv->cache_valid = FALSE;
 }
 
+/*
+  Wherefore this function?
+  Formerly, the above defined callback functions were called after
+  reading data from file or from clipboard, in both cases there was neither
+  a change in flip nor rotate, but the above functions only work for
+  data which has changed. The "Not Changed, Just Redraw" was missing here.
+ */
+static void
+part_arrange_canvas_item (ItemData *data, SheetItem *sheet_item)
+{
+  double affine[6];
+	double affine_scale[6];
+	double affine_rotate[6];
+	GList *list;
+	GSList *label_items;
+	GtkAnchorType anchor;
+	GnomeCanvasGroup *group;
+	GnomeCanvasItem *canvas_item;
+	PartItem *item;
+	PartItemPriv *priv;
+	Part *part;
+	int angle;
+
+	g_return_if_fail (sheet_item != NULL);
+	g_return_if_fail (IS_PART_ITEM (sheet_item));
+
+	item = PART_ITEM (sheet_item);
+	group = GNOME_CANVAS_GROUP (item);
+	part = PART(data);
+
+	priv = item->priv;
+	
+	angle=part_get_rotation(part);
+
+	if (angle!=0) {
+	  if ((part->priv->flip & ID_FLIP_HORIZ)
+	      && (part->priv->flip & ID_FLIP_VERT)) {
+	    art_affine_rotate(affine,angle+180);
+	  } else { if ((part->priv->flip & ID_FLIP_HORIZ)
+		       || (part->priv->flip & ID_FLIP_VERT)) {
+	      if (part->priv->flip & ID_FLIP_HORIZ)
+		art_affine_scale(affine_scale,-1,1);
+	      else
+		art_affine_scale(affine_scale,1,-1);
+
+	      art_affine_rotate(affine_rotate,angle);
+
+	      matrix_mult(affine_rotate,affine_scale,affine);
+	      affine[4]=affine_rotate[4];
+	      affine[5]=affine_rotate[5];
+	    } else
+	      art_affine_rotate (affine, angle);
+	  }
+	} else {
+	  art_affine_scale(affine,1,1); //default
+	  if ((part->priv->flip & ID_FLIP_HORIZ)
+	      && (part->priv->flip & ID_FLIP_VERT))
+	    art_affine_scale(affine,-1,-1);
+	  if ((part->priv->flip & ID_FLIP_HORIZ)
+	      && !(part->priv->flip & ID_FLIP_VERT))
+	    art_affine_scale(affine,-1,1);
+	  if (!(part->priv->flip & ID_FLIP_HORIZ)
+	      && (part->priv->flip & ID_FLIP_VERT))
+	    art_affine_scale(affine,1,-1);
+	}
+
+	for (list = group->item_list; list; list = list->next) {
+		canvas_item = GNOME_CANVAS_ITEM (list->data);
+		gnome_canvas_item_affine_relative (canvas_item, affine);
+	}
+
+	/*
+	 * Get the right anchor for the labels. This is needed since the
+	 * canvas don't know how to rotate text and since we rotate the
+	 * label_group instead of the labels directly.
+	 */
+	switch (part_get_rotation (part)) {
+	case 0:
+		anchor = GTK_ANCHOR_SOUTH_WEST;
+		break;
+	case 90:
+		anchor = GTK_ANCHOR_NORTH_WEST;
+		break;
+	case 180:
+		anchor = GTK_ANCHOR_NORTH_EAST;
+		break;
+	case 270:
+		anchor = GTK_ANCHOR_SOUTH_EAST;
+		break;
+	default:
+		anchor = GTK_ANCHOR_SOUTH_WEST;
+		break;
+	}
+
+
+	for (label_items = priv->label_items; label_items;
+	     label_items = label_items->next) {
+		gnome_canvas_item_set (
+			GNOME_CANVAS_ITEM (label_items->data),
+			"anchor", anchor,
+			NULL);
+	}
+
+	for (label_items = priv->label_nodes; label_items;
+	     label_items = label_items->next) {
+		gnome_canvas_item_set (
+			GNOME_CANVAS_ITEM (label_items->data),
+			"anchor", anchor,
+			NULL);
+	}
+
+	/*
+	 * Invalidate the bounding box cache.
+	 */
+	priv->cache_valid = FALSE;
+}
+
 void
 part_item_signal_connect_floating_group (Sheet *sheet, SchematicView *sv)
 {
@@ -1228,8 +1373,6 @@ part_item_new_from_part (Sheet *sheet, Part *part)
 	LibraryPart *library_part;
 	PartPriv *priv;
 	PartItem *item;
-	int angle;
-	IDFlip flip;
 
 	priv = part->priv;
 
@@ -1245,18 +1388,7 @@ part_item_new_from_part (Sheet *sheet, Part *part)
 	create_canvas_labels (item, part);
 	create_canvas_label_nodes(item, part);
 
-	angle = part_get_rotation (part);
-	part_rotated_callback (ITEM_DATA (part), angle, SHEET_ITEM (item));
-
-	flip = part_get_flip (part);
-	if (flip & ID_FLIP_HORIZ)
-		part_flipped_callback (ITEM_DATA (part),
-			TRUE, SHEET_ITEM (item));
-
-	if (flip & ID_FLIP_VERT)
-		part_flipped_callback (ITEM_DATA (part),
-			FALSE, SHEET_ITEM (item));
-
+	part_arrange_canvas_item(ITEM_DATA (part), SHEET_ITEM (item));
 	return item;
 }
 
