@@ -10,7 +10,7 @@
  * Web page: http://arrakis.lug.fi.uba.ar/
  *
  * Copyright (C) 1999-2001  Richard Hult
- * Copyright (C) 2003,2006  LUGFI
+ * Copyright (C) 2003,2006  Ricardo Markiewicz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -93,13 +93,13 @@ typedef struct {
 static TextboxPropDialog *prop_dialog = NULL;
 static SheetItemClass *textbox_item_parent_class = NULL;
 
-static GnomeUIInfo textbox_popup_menu [] = {
-	GNOMEUIINFO_SEPARATOR,
-
-	{ GNOME_APP_UI_ITEM, N_("Edit..."), N_("Edit the text"), edit_cmd,
-	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_PROPERTIES, 0, 0 },
-
-	GNOMEUIINFO_END };
+/* Use EDIT!! */
+static const char *textbox_item_context_menu =
+"<ui>"
+"  <popup name='ItemMenu'>"
+"    <menuitem action='EditText'/>"
+"  </popup>"
+"</ui>";
 
 enum {
 	TEXTBOX_ITEM_ARG_0,
@@ -173,11 +173,6 @@ textbox_item_class_init (TextboxItemClass *textbox_item_class)
 	sheet_item_class->edit_properties = edit_textbox;
 	sheet_item_class->place = textbox_item_place;
 	sheet_item_class->place_ghost = textbox_item_place_ghost;
-
-	sheet_item_class->context_menu = g_new0 (SheetItemMenu, 1);
-	sheet_item_class->context_menu->menu = textbox_popup_menu;
-	sheet_item_class->context_menu->size =
-		sizeof (textbox_popup_menu) / sizeof (textbox_popup_menu[0]);
 }
 
 static void
@@ -190,6 +185,8 @@ textbox_item_init (TextboxItem *item)
 
 	priv->highlight = FALSE;
 	priv->cache_valid = FALSE;
+
+	sheet_item_add_menu (SHEET_ITEM (item), textbox_item_context_menu);
 }
 
 /*static void
@@ -613,6 +610,20 @@ create_textbox_event (Sheet *sheet, GdkEvent *event, SchematicView *sv)
 	return TRUE;
 }
 
+	void
+textbox_item_cancel_listen (SchematicView *sv)
+{
+	Sheet *sheet;
+
+	g_return_if_fail (sv != NULL);
+	g_return_if_fail (IS_SCHEMATIC_VIEW (sv));
+
+	sheet = schematic_view_get_sheet (sv);
+
+	sheet->state = SHEET_STATE_NONE;
+	g_signal_handlers_disconnect_by_func (G_OBJECT (sheet), G_CALLBACK(create_textbox_event), sv);
+}
+
 void
 textbox_item_listen (SchematicView *sv)
 {
